// gpibExample.cpp
// CPlusPlus Non VISA GPIB IO to Thermotron Chamber Controllers
// This example built with Microsoft Visual Studio 2012 using National Instruments IEEE488.2 driver

// Be sure to add the following path to Project properties:
// Configuration Properties -> C/C++ -> General -> Additional Include Directories:
// C:\Program Files (x86)\National Instruments\Shared\ExternalCompilerSupport\C\include
// Configuration Properties -> Linker -> Input -> Additional Dependencies: ni4882.obj
// Configuration Properties -> Linker -> General -> Additional Library Directories:
// C:\Program Files (x86)\National Instruments\Shared\ExternalCompilerSupport\C\lib32\msvc

#include "stdafx.h"
#include "windows.h"
#include "decl-32.h"

#define ERRMSGSIZE 1024 // Maximum size of command string
#define ARRAYSIZE 1024 // Size of read buffer

#define GPIBBOARDADDR 0 // Board Index of GPIB board
#define CONTROLLER_GPIB_ADDR 10 // GPIB address of CHAMBER controller
#define NO_SECONDARY_ADDR 0 // no Secondary address
#define TIMEOUT T10s // Timeout value = 10 seconds
#define EOTMODE 1 // Enable the END message
#define EOSMODE 1 // Enable the EOS (End Of String) mode

int nDevice;
char ValueStr[ARRAYSIZE + 1];
char writePtr[ARRAYSIZE];
char ErrorMnemonic[21][5] = {"EDVR", "ECIC", "ENOL", "EADR", "EARG",
  "ESAC", "EABO", "ENEB", "EDMA", "",
  "EOIP", "ECAP", "EFSO", "", "EBUS",
  "ESTB", "ESRQ", "", "", "", "ETAB"};


void GPIBWrite(char* cmdPtr);
char *GPIBRead(void);
void GPIBCleanup(int nDev, char* ErrorMsgPtr);

int _tmain(int argc, _TCHAR* argv[])
{

char *resultPtr;
char *result;
char *value;

 /*=========================================
 INITIALIZATION SECTION
 =========================================
 The application brings the nDevice online using ibdev. A device handle, nDevice, is returned and is used in all subsequent calls to the Device.*/

  nDevice = ibdev(GPIBBOARDADDR, CONTROLLER_GPIB_ADDR, NO_SECONDARY_ADDR, TIMEOUT, EOTMODE, EOSMODE);
  if (ibsta & ERR)
  {
    printf("Unable to open handle to nDevice/nibsta = 0x%x iberr = %d/n", ibsta, iberr);
    return 1;
  }

  /* Do a GPIB Clear of the nDevice. If the error bit ERR is set in ibsta, call GPIBCleanup with an error message. */
  ibclr (nDevice);
  if (ibsta & ERR)
  {
    GPIBCleanup(nDevice, "Unable to perform GPIB clear of the Device");
    return 1;
  }


 /*=========================================
 MAIN BODY SECTION
 =========================================*/

  Sleep(2000); // Give initialization time to complete
  GPIBWrite("IDEN?\r");
  printf("Write IDEN? message to controller\n");
  resultPtr = GPIBRead();
  printf("Result= %s\n", resultPtr);

  GPIBWrite("RUNM\r");
  printf("Write RUNM message to controller\n");
  // Be sure to always read after a write.  Even if not interested in returning data.  If write only command, status is returned.
  resultPtr = GPIBRead();

  int nBaseTemperature = 25;
  for (int i = 0; i < 10; i++)
  {
    Sleep(1000);
    GPIBWrite("PVAR1?\r");
    printf("\nLoop Count=%d Write PVAR1? to controller\n", i);
    resultPtr = GPIBRead();
    printf("Result= %s\n", resultPtr);

    sprintf(writePtr,"SETP1,%d\r",nBaseTemperature + i);
    GPIBWrite(writePtr);
    printf("\nWrite SETP1,%d to controller\n", nBaseTemperature + i);
    resultPtr = GPIBRead();

    GPIBWrite("SETP1?\r");
    printf("\nWrite SETP1? to controller\n");
    resultPtr = GPIBRead();
    printf("Result= %s\n", resultPtr);
  }

  GPIBWrite("STOP\r");
  printf("\nWrite STOP message to controller\n");
  resultPtr = GPIBRead();

/*=========================================
  CLEANUP SECTION
  =========================================*/

  /* The Device is returned to front panel control. */
  ibonl(nDevice, 0);

  printf("\nPress <Enter> to continue");
  int nChar = getchar();
	return 0;
}


/* Write to the nDevice*/
void GPIBWrite(char* cmdPtr)
{
int length;
char ErrorMsg[ERRMSGSIZE + 1];
length = strlen(cmdPtr) ;

  ibwrt (nDevice, cmdPtr, length);
  if (ibsta & ERR)
  {
  strcpy(ErrorMsg, "Unable to write this command to Device:/n");
  strcat(ErrorMsg, cmdPtr);

  GPIBCleanup(nDevice, ErrorMsg);
  exit(1);
  }
}

/* Read from the nDevice */
char* GPIBRead(void)
{
  memset(ValueStr,0,sizeof(ValueStr));  // clear read buffer
  ibrd (nDevice, ValueStr, ARRAYSIZE);
  if (ibsta & ERR)
  {
    GPIBCleanup(nDevice, "Unable to read from the nDevice");
    exit(1);
  }
else
  return ValueStr;
}

/* After each GPIB call, the application checks whether the call succeeded. If an NI-488.2 call fails,
   the GPIB driver sets the corresponding bit in the global status variable. If the call failed,
   this procedure prints an error message, takes the nDevice offline and exits. */
void GPIBCleanup(int Dev, char* ErrorMsg)
{
  printf("Error : %s/nibsta = 0x%x iberr = %d (%s)/n", ErrorMsg, ibsta, iberr, ErrorMnemonic[iberr]);
  if (Dev != -1)
  {
    printf("Cleanup/n");
    ibonl (Dev, 0);
  }
}
